@@ -10,7 +10,7 @@ class IndexCounter:
 
     @index.setter
     def index(self, value: int) -> None:
-        if not (0 < value < self._length):
+        if not (0 <= value < self._length):
             raise IndexError(f"New index is not within valid range of [0 to {self._length}).")
 
         self._index = value
@@ -22,6 +22,8 @@ class IndexCounter:
     def step(self, by: int) -> None:
         index = self._index + by
         if self.circular:
+            if self._length == 1:
+                index = 0
             if index >= self._length:
                 index %= self._length
             elif index < 0:
